<?php
// An individual demand post attached to a particular category, zip code and user. May also be
//   attached to a supplier.
class Post extends AppModel {
  public $name = 'Post';
  public $displayField = 'title';
  public $recursive = 2;
  
  // The number of days the timers for each countdown last.
  const pledgeCountdown =  1; // days FIXME: Change pledge countdown to 10 days in final version.
  const bidCountdown    =  1; // days FIXME: Change bid countdown to 4 days in final version.
  
  // This information is also stored in the votes controller, so we should find a way to eliminate
  //   that duplication.
  const voteUp   =  1;
  const voteDown = -1;
  
  // The number of posts to retrieve when getting the top posts for a given period.
  const topPostCount = 3;
  
  public $validate = array(
    'title' => array( 'notempty' => array(
      'rule' => array( 'notempty' ),
      'message' => 'You must enter a title for your demand post.' )
    ),
    
    'introduction' => array( 'notempty' => array(
      'rule' => array( 'notempty' ),
      'message' => 'You must enter a short introduction for your demand post.' )
    ),
    
    'body' => array( 'notempty' => array(
      'rule' => array( 'notempty' ),
      'message' => 'The detailed description of your demand post cannot be empty.' )
    ),
  );

  public $belongsTo = array(
    'User',
    'PostCategory' => array( 'counterCache' => true ),
    'ZipCode'      => array( 'counterCache' => true )
  );

  public $hasMany = array( 'Comment', 'Vote', 'MediaFile' );
  
//==================================================================================================
function __PUBLIC_METHODS() {}
//==================================================================================================

//==================================================================================================
  // Sets the timer given the typer type to the amount specified in the appropriate constant.
  public function set_timer( $timerType ) {
    $post = $this->_get_this_post();

    if ( $post['Post']['timer'] != $timerType ) {
      $days = ( $timerType == 'pledge' ) ? self::pledgeCountdown : self::bidCountdown;
      $timer = date( 'Y-m-d H:i:s', strtotime( '+' . $days . ' days' ) );
      
      $this->saveField( 'timer', $timer );
      $this->saveField( 'timer_type', $timerType );      
    }
  }

//==================================================================================================
  // Clear the timer of all relevant values.
  public function clear_timer() {
    $this->saveField( 'timer', '' );
    $this->saveField( 'timer_type', '' );
  }

//==================================================================================================
  // Construct a string that specifies in human-readable format how much time is left on the pledge
  //   timer.
  public function pledge_time_left() {
    $post = $this->_get_this_post();
    return duration_to_string( strtotime( $post['Post']['timer'] ) - time() );
  }

//==================================================================================================
  // Construct a string that specifies in human-readable format how much time is left on the bid
  //   timer.
  public function bid_time_left() {
    $post = $this->_get_this_post();
    return duration_to_string( strtotime( $post['Post']['timer'] ) - time() );
  }
  
//==================================================================================================
  // Construct a string that specifies what the vote up count is for this post.
  public function votes_up_str() {
    $votesUp = $this->_votes_count( self::voteUp );
    return $votesUp . ( ( $votesUp == 1 ) ? ' Vote Up' : ' Votes Up' );
  }

//==================================================================================================
  // Construct a string that specifies what the vote down count is for this post.
  public function votes_down_str() {
    $votesDown = $this->_votes_count( self::voteDown );
    return $votesDown . ( ( $votesDown == 1 ) ? ' Vote Down' : ' Votes Down' );
  }

//==================================================================================================
  // Returns the most recent posts in the system, the number returned defined by topPostCount
  public function most_recent_posts() {
    return $this->find( 'all', array( 'order' => array('Post.created DESC' ),
                                      'limit' => self::topPostCount ) );
  }

//==================================================================================================
  // Returns the top demands for the given period, the number of which that are returned determined
  //   by this object's constant.
  public function top_demands( $period ) {
    $posts = $this->find( 'all', array(
      'conditions' => array( 'Post.created >' => date( 'Y-m-d H:i:s', strtotime( '-1 ' . $period ) ) ),
      'limit'      => self::topPostCount ) );
    return $posts;
  }

//==================================================================================================
function __PRIVATE_METHODS() {}
//==================================================================================================

//==================================================================================================
  // Return this post's complete record.
  private function _get_this_post() {
    // FIXME: Find out if this is really necessary.
    return  $this->find( 'first', array( 'conditions' => array( 'Post.id' => $this->id ) ) );
  }

//==================================================================================================
  // Manually calculate the vote count for this post, which can probably be eliminated when we're
  //   sure that the vote_score field is being updated reliably.
  private function _votes_count( $direction ) {
    $voteCount = 0;
    $votes = $this->Vote->find( 'all', array( 'conditions' => array( 'post_id' => $this->id ) ) );
    
    if ( !empty( $votes ) ) {
      foreach ( $votes as $vote ) {
        $voteCount += ( $vote['Vote']['direction'] == $direction ) ? 1 : 0;
      }
    }
    
    return $voteCount;
  }

}
?>